@@ -16,9 +16,17 @@ from bids.utils import listify
 
 from nipype import logging
 from nipype.interfaces.base import (
-    traits, isdefined, Undefined,
-    TraitedSpec, BaseInterfaceInputSpec, DynamicTraitedSpec,
-    File, Directory, InputMultiObject, OutputMultiObject, Str,
+    traits,
+    isdefined,
+    Undefined,
+    TraitedSpec,
+    BaseInterfaceInputSpec,
+    DynamicTraitedSpec,
+    File,
+    Directory,
+    InputMultiObject,
+    OutputMultiObject,
+    Str,
     SimpleInterface,
 )
 from nipype.interfaces.io import add_traits
@@ -28,14 +36,12 @@ from ..utils.images import overwrite_header
 from ..utils.misc import splitext as _splitext, _copy_any
 
 regz = re.compile(r"\.gz$")
-_pybids_spec = loads(
-    Path(_pkgres("niworkflows", "data/nipreps.json")).read_text()
-)
+_pybids_spec = loads(Path(_pkgres("niworkflows", "data/nipreps.json")).read_text())
 BIDS_DERIV_ENTITIES = frozenset({e["name"] for e in _pybids_spec["entities"]})
 BIDS_DERIV_PATTERNS = tuple(_pybids_spec["default_path_patterns"])
 
 STANDARD_SPACES = _get_template_list()
-LOGGER = logging.getLogger('nipype.interface')
+LOGGER = logging.getLogger("nipype.interface")
 
 
 def _none():
@@ -43,22 +49,26 @@ def _none():
 
 
 # Automatically coerce certain suffixes (DerivativesDataSink)
-DEFAULT_DTYPES = defaultdict(_none, (
-    ("mask", "uint8"),
-    ("dseg", "int16"),
-    ("probseg", "float32"),
-    ("boldref", "source"))
+DEFAULT_DTYPES = defaultdict(
+    _none,
+    (
+        ("mask", "uint8"),
+        ("dseg", "int16"),
+        ("probseg", "float32"),
+        ("boldref", "source"),
+    ),
 )
 
 
 class _BIDSBaseInputSpec(BaseInterfaceInputSpec):
     bids_dir = traits.Either(
-        (None, Directory(exists=True)), usedefault=True, desc='optional bids directory')
-    bids_validate = traits.Bool(True, usedefault=True, desc='enable BIDS validator')
+        (None, Directory(exists=True)), usedefault=True, desc="optional bids directory"
+    )
+    bids_validate = traits.Bool(True, usedefault=True, desc="enable BIDS validator")
 
 
 class _BIDSInfoInputSpec(_BIDSBaseInputSpec):
-    in_file = File(mandatory=True, desc='input file, part of a BIDS tree')
+    in_file = File(mandatory=True, desc="input file, part of a BIDS tree")
 
 
 class _BIDSInfoOutputSpec(DynamicTraitedSpec):
@@ -168,8 +178,10 @@ sub-01/func/ses-retest/sub-01_ses-retest_task-covertverbgeneration_bold.nii.gz''
             except ValueError:
                 pass
         params = parse_file_entities(in_file)
-        self._results = {key: params.get(key, Undefined)
-                         for key in _BIDSInfoOutputSpec().get().keys()}
+        self._results = {
+            key: params.get(key, Undefined)
+            for key in _BIDSInfoOutputSpec().get().keys()
+        }
         return runtime
 
 
@@ -179,14 +191,14 @@ class _BIDSDataGrabberInputSpec(BaseInterfaceInputSpec):
 
 
 class _BIDSDataGrabberOutputSpec(TraitedSpec):
-    out_dict = traits.Dict(desc='output data structure')
-    fmap = OutputMultiObject(desc='output fieldmaps')
-    bold = OutputMultiObject(desc='output functional images')
-    sbref = OutputMultiObject(desc='output sbrefs')
-    t1w = OutputMultiObject(desc='output T1w images')
-    roi = OutputMultiObject(desc='output ROI images')
-    t2w = OutputMultiObject(desc='output T2w images')
-    flair = OutputMultiObject(desc='output FLAIR images')
+    out_dict = traits.Dict(desc="output data structure")
+    fmap = OutputMultiObject(desc="output fieldmaps")
+    bold = OutputMultiObject(desc="output functional images")
+    sbref = OutputMultiObject(desc="output sbrefs")
+    t1w = OutputMultiObject(desc="output T1w images")
+    roi = OutputMultiObject(desc="output ROI images")
+    t2w = OutputMultiObject(desc="output T2w images")
+    flair = OutputMultiObject(desc="output FLAIR images")
 
 
 class BIDSDataGrabber(SimpleInterface):
@@ -209,7 +221,7 @@ class BIDSDataGrabber(SimpleInterface):
     _require_funcs = True
 
     def __init__(self, *args, **kwargs):
-        anat_only = kwargs.pop('anat_only')
+        anat_only = kwargs.pop("anat_only")
         super(BIDSDataGrabber, self).__init__(*args, **kwargs)
         if anat_only is not None:
             self._require_funcs = not anat_only
@@ -217,52 +229,66 @@ class BIDSDataGrabber(SimpleInterface):
     def _run_interface(self, runtime):
         bids_dict = self.inputs.subject_data
 
-        self._results['out_dict'] = bids_dict
+        self._results["out_dict"] = bids_dict
         self._results.update(bids_dict)
 
-        if not bids_dict['t1w']:
-            raise FileNotFoundError('No T1w images found for subject sub-{}'.format(
-                self.inputs.subject_id))
+        if not bids_dict["t1w"]:
+            raise FileNotFoundError(
+                "No T1w images found for subject sub-{}".format(self.inputs.subject_id)
+            )
 
-        if self._require_funcs and not bids_dict['bold']:
-            raise FileNotFoundError('No functional images found for subject sub-{}'.format(
-                self.inputs.subject_id))
+        if self._require_funcs and not bids_dict["bold"]:
+            raise FileNotFoundError(
+                "No functional images found for subject sub-{}".format(
+                    self.inputs.subject_id
+                )
+            )
 
-        for imtype in ['bold', 't2w', 'flair', 'fmap', 'sbref', 'roi']:
+        for imtype in ["bold", "t2w", "flair", "fmap", "sbref", "roi"]:
             if not bids_dict[imtype]:
-                LOGGER.info('No "%s" images found for sub-%s',
-                            imtype, self.inputs.subject_id)
+                LOGGER.info(
+                    'No "%s" images found for sub-%s', imtype, self.inputs.subject_id
+                )
 
         return runtime
 
 
 class _DerivativesDataSinkInputSpec(DynamicTraitedSpec, BaseInterfaceInputSpec):
     base_directory = traits.Directory(
-        desc='Path to the base directory for storing data.')
-    check_hdr = traits.Bool(True, usedefault=True, desc='fix headers of NIfTI outputs')
+        desc="Path to the base directory for storing data."
+    )
+    check_hdr = traits.Bool(True, usedefault=True, desc="fix headers of NIfTI outputs")
     compress = InputMultiObject(
-        traits.Either(None, traits.Bool), usedefault=True,
+        traits.Either(None, traits.Bool),
+        usedefault=True,
         desc="whether ``in_file`` should be compressed (True), uncompressed (False) "
-             "or left unmodified (None, default).")
-    data_dtype = Str(desc='NumPy datatype to coerce NIfTI data to, or `source` to'
-                          'match the input file dtype')
+        "or left unmodified (None, default).",
+    )
+    data_dtype = Str(
+        desc="NumPy datatype to coerce NIfTI data to, or `source` to"
+        "match the input file dtype"
+    )
     dismiss_entities = InputMultiObject(
-        traits.Either(None, Str), usedefault=True,
-        desc="a list entities that will not be propagated from the source file")
-    in_file = InputMultiObject(File(exists=True), mandatory=True,
-                               desc='the object to be saved')
-    meta_dict = traits.DictStrAny(desc='an input dictionary containing metadata')
-    source_file = File(exists=False, mandatory=True, desc='the input func file')
+        traits.Either(None, Str),
+        usedefault=True,
+        desc="a list entities that will not be propagated from the source file",
+    )
+    in_file = InputMultiObject(
+        File(exists=True), mandatory=True, desc="the object to be saved"
+    )
+    meta_dict = traits.DictStrAny(desc="an input dictionary containing metadata")
+    source_file = File(exists=False, mandatory=True, desc="the input func file")
 
 
 class _DerivativesDataSinkOutputSpec(TraitedSpec):
-    out_file = OutputMultiObject(File(exists=True, desc='written file path'))
-    out_meta = OutputMultiObject(File(exists=True, desc='written JSON sidecar path'))
+    out_file = OutputMultiObject(File(exists=True, desc="written file path"))
+    out_meta = OutputMultiObject(File(exists=True, desc="written JSON sidecar path"))
     compression = OutputMultiObject(
         traits.Either(None, traits.Bool),
-        desc='whether ``in_file`` should be compressed (True), uncompressed (False) '
-             'or left unmodified (None).')
-    fixed_hdr = traits.List(traits.Bool, desc='whether derivative header was fixed')
+        desc="whether ``in_file`` should be compressed (True), uncompressed (False) "
+        "or left unmodified (None).",
+    )
+    fixed_hdr = traits.List(traits.Bool, desc="whether derivative header was fixed")
 
 
 class DerivativesDataSink(SimpleInterface):
@@ -412,13 +438,16 @@ space-MNI152NLin6Asym_desc-preproc_bold.json'
 
     def __init__(self, allowed_entities=None, out_path_base=None, **inputs):
         """Initialize the SimpleInterface and extend inputs with custom entities."""
-        self._allowed_entities = set(allowed_entities or []).union(self._allowed_entities)
+        self._allowed_entities = set(allowed_entities or []).union(
+            self._allowed_entities
+        )
         if out_path_base:
             self.out_path_base = out_path_base
 
         self._metadata = {}
-        self._static_traits = self.input_spec.class_editable_traits() \
-            + sorted(self._allowed_entities)
+        self._static_traits = self.input_spec.class_editable_traits() + sorted(
+            self._allowed_entities
+        )
         for dynamic_input in set(inputs) - set(self._static_traits):
             self._metadata[dynamic_input] = inputs.pop(dynamic_input)
 
@@ -458,7 +487,8 @@ space-MNI152NLin6Asym_desc-preproc_bold.json'
         # Override extension with that of the input file(s)
         out_entities["extension"] = [
             # _splitext does not accept .surf.gii (for instance)
-            "".join(Path(orig_file).suffixes).lstrip(".") for orig_file in in_file
+            "".join(Path(orig_file).suffixes).lstrip(".")
+            for orig_file in in_file
         ]
 
         compress = listify(self.inputs.compress) or [None]
@@ -488,13 +518,15 @@ space-MNI152NLin6Asym_desc-preproc_bold.json'
         if custom_entities:
             # Example: f"{key}-{{{key}}}" -> "task-{task}"
             custom_pat = "_".join(f"{key}-{{{key}}}" for key in sorted(custom_entities))
-            patterns = [pat.replace("_{suffix", "_".join(('', custom_pat, "{suffix")))
-                        for pat in patterns]
+            patterns = [
+                pat.replace("_{suffix", "_".join(("", custom_pat, "{suffix")))
+                for pat in patterns
+            ]
 
         # Prepare SimpleInterface outputs object
-        self._results['out_file'] = []
-        self._results['compression'] = []
-        self._results['fixed_hdr'] = [False] * len(in_file)
+        self._results["out_file"] = []
+        self._results["compression"] = []
+        self._results["fixed_hdr"] = [False] * len(in_file)
 
         dest_files = build_path(out_entities, path_patterns=patterns)
         if not dest_files:
@@ -503,17 +535,20 @@ space-MNI152NLin6Asym_desc-preproc_bold.json'
         # Make sure the interpolated values is embedded in a list, and check
         dest_files = listify(dest_files)
         if len(in_file) != len(dest_files):
-            raise ValueError(f"Input files ({len(in_file)}) not matched "
-                             f"by interpolated patterns ({len(dest_files)}).")
+            raise ValueError(
+                f"Input files ({len(in_file)}) not matched "
+                f"by interpolated patterns ({len(dest_files)})."
+            )
 
         for i, (orig_file, dest_file) in enumerate(zip(in_file, dest_files)):
             out_file = out_path / dest_file
             out_file.parent.mkdir(exist_ok=True, parents=True)
-            self._results['out_file'].append(str(out_file))
-            self._results['compression'].append(_copy_any(orig_file, str(out_file)))
+            self._results["out_file"].append(str(out_file))
+            self._results["compression"].append(_copy_any(orig_file, str(out_file)))
 
-            is_nifti = out_file.name.endswith(('.nii', '.nii.gz')) \
-                and not out_file.name.endswith(('.dtseries.nii', '.dtseries.nii.gz'))
+            is_nifti = out_file.name.endswith(
+                (".nii", ".nii.gz")
+            ) and not out_file.name.endswith((".dtseries.nii", ".dtseries.nii.gz"))
             data_dtype = self.inputs.data_dtype or DEFAULT_DTYPES[self.inputs.suffix]
             if is_nifti and any((self.inputs.check_hdr, data_dtype)):
                 # Do not use mmap; if we need to access the data at all, it will be to
@@ -522,22 +557,24 @@ space-MNI152NLin6Asym_desc-preproc_bold.json'
 
                 if self.inputs.check_hdr:
                     hdr = nii.header
-                    curr_units = tuple([None if u == 'unknown' else u
-                                        for u in hdr.get_xyzt_units()])
-                    curr_codes = (int(hdr['qform_code']), int(hdr['sform_code']))
+                    curr_units = tuple(
+                        [None if u == "unknown" else u for u in hdr.get_xyzt_units()]
+                    )
+                    curr_codes = (int(hdr["qform_code"]), int(hdr["sform_code"]))
 
                     # Default to mm, use sec if data type is bold
                     units = (
-                        curr_units[0] or 'mm',
-                        'sec' if out_entities["suffix"] == 'bold' else None
+                        curr_units[0] or "mm",
+                        "sec" if out_entities["suffix"] == "bold" else None,
                     )
                     xcodes = (1, 1)  # Derivative in its original scanner space
                     if self.inputs.space:
-                        xcodes = (4, 4) if self.inputs.space in STANDARD_SPACES \
-                            else (2, 2)
+                        xcodes = (
+                            (4, 4) if self.inputs.space in STANDARD_SPACES else (2, 2)
+                        )
 
                     if curr_codes != xcodes or curr_units != units:
-                        self._results['fixed_hdr'][i] = True
+                        self._results["fixed_hdr"][i] = True
                         hdr.set_qform(nii.affine, xcodes[0])
                         hdr.set_sform(nii.affine, xcodes[1])
                         hdr.set_xyzt_units(*units)
@@ -545,7 +582,7 @@ space-MNI152NLin6Asym_desc-preproc_bold.json'
                         # Rewrite file with new header
                         overwrite_header(nii, out_file)
 
-                if data_dtype == 'source':  # match source dtype
+                if data_dtype == "source":  # match source dtype
                     try:
                         data_dtype = nb.load(self.inputs.source_file).get_data_dtype()
                     except Exception:
@@ -559,25 +596,41 @@ space-MNI152NLin6Asym_desc-preproc_bold.json'
                         # load updated NIfTI
                         nii = nb.load(out_file, mmap=False)
                     data_dtype = np.dtype(data_dtype)
-                    if nii.get_data_dtype() != data_dtype:
+                    orig_dtype = nii.get_data_dtype()
+                    if orig_dtype != data_dtype:
+                        LOGGER.warning(
+                            f"Changing {out_file} dtype from {orig_dtype} to {data_dtype}"
+                        )
+                        # coerce dataobj to new data dtype
+                        if np.issubdtype(data_dtype, np.integer):
+                            new_data = np.rint(nii.dataobj).astype(data_dtype)
+                        else:
+                            new_data = data_dtype(nii.dataobj)
+                        # and set header to match
                         nii.set_data_dtype(data_dtype)
+                        nii = nii.__class__(new_data, nii.affine, nii.header)
                         nii.to_filename(out_file)
 
-        if len(self._results['out_file']) == 1:
+        if len(self._results["out_file"]) == 1:
             meta_fields = self.inputs.copyable_trait_names()
-            self._metadata.update({
-                k: getattr(self.inputs, k)
-                for k in meta_fields if k not in self._static_traits})
+            self._metadata.update(
+                {
+                    k: getattr(self.inputs, k)
+                    for k in meta_fields
+                    if k not in self._static_traits
+                }
+            )
             if self._metadata:
-                sidecar = (Path(self._results['out_file'][0]).parent
-                           / ('%s.json' % _splitext(self._results['out_file'][0])[0]))
+                sidecar = Path(self._results["out_file"][0]).parent / (
+                    "%s.json" % _splitext(self._results["out_file"][0])[0]
+                )
                 sidecar.write_text(dumps(self._metadata, sort_keys=True, indent=2))
-                self._results['out_meta'] = str(sidecar)
+                self._results["out_meta"] = str(sidecar)
         return runtime
 
 
 class _ReadSidecarJSONInputSpec(_BIDSBaseInputSpec):
-    in_file = File(exists=True, mandatory=True, desc='the input nifti file')
+    in_file = File(exists=True, mandatory=True, desc="the input nifti file")
 
 
 class _ReadSidecarJSONOutputSpec(_BIDSInfoOutputSpec):
@@ -642,48 +695,54 @@ class ReadSidecarJSON(SimpleInterface):
 
     def _run_interface(self, runtime):
         self.layout = self.inputs.bids_dir or self.layout
-        self.layout = _init_layout(self.inputs.in_file,
-                                   self.layout,
-                                   self.inputs.bids_validate)
+        self.layout = _init_layout(
+            self.inputs.in_file, self.layout, self.inputs.bids_validate
+        )
 
         # Fill in BIDS entities of the output ("*_id")
         output_keys = list(_BIDSInfoOutputSpec().get().keys())
         params = self.layout.parse_file_entities(self.inputs.in_file)
-        self._results = {key: params.get(key.split('_')[0], Undefined)
-                         for key in output_keys}
+        self._results = {
+            key: params.get(key.split("_")[0], Undefined) for key in output_keys
+        }
 
         # Fill in metadata
         metadata = self.layout.get_metadata(self.inputs.in_file)
-        self._results['out_dict'] = metadata
+        self._results["out_dict"] = metadata
 
         # Set dynamic outputs if fields input is present
         for fname in self._fields:
             if not self._undef_fields and fname not in metadata:
                 raise KeyError(
-                    'Metadata field "%s" not found for file %s' % (
-                        fname, self.inputs.in_file))
+                    'Metadata field "%s" not found for file %s'
+                    % (fname, self.inputs.in_file)
+                )
             self._results[fname] = metadata.get(fname, Undefined)
         return runtime
 
 
 class _BIDSFreeSurferDirInputSpec(BaseInterfaceInputSpec):
-    derivatives = Directory(exists=True, mandatory=True,
-                            desc='BIDS derivatives directory')
-    freesurfer_home = Directory(exists=True, mandatory=True,
-                                desc='FreeSurfer installation directory')
-    subjects_dir = traits.Either(traits.Str(),
-                                 Directory(),
-                                 default='freesurfer',
-                                 usedefault=True,
-                                 desc='Name of FreeSurfer subjects directory')
-    spaces = traits.List(traits.Str, desc='Set of output spaces to prepare')
-    overwrite_fsaverage = traits.Bool(False, usedefault=True,
-                                      desc='Overwrite fsaverage directories, if present')
+    derivatives = Directory(
+        exists=True, mandatory=True, desc="BIDS derivatives directory"
+    )
+    freesurfer_home = Directory(
+        exists=True, mandatory=True, desc="FreeSurfer installation directory"
+    )
+    subjects_dir = traits.Either(
+        traits.Str(),
+        Directory(),
+        default="freesurfer",
+        usedefault=True,
+        desc="Name of FreeSurfer subjects directory",
+    )
+    spaces = traits.List(traits.Str, desc="Set of output spaces to prepare")
+    overwrite_fsaverage = traits.Bool(
+        False, usedefault=True, desc="Overwrite fsaverage directories, if present"
+    )
 
 
 class _BIDSFreeSurferDirOutputSpec(TraitedSpec):
-    subjects_dir = traits.Directory(exists=True,
-                                    desc='FreeSurfer subjects directory')
+    subjects_dir = traits.Directory(exists=True, desc="FreeSurfer subjects directory")
 
 
 class BIDSFreeSurferDir(SimpleInterface):
@@ -716,9 +775,9 @@ class BIDSFreeSurferDir(SimpleInterface):
         if not subjects_dir.is_absolute():
             subjects_dir = Path(self.inputs.derivatives) / subjects_dir
         subjects_dir.mkdir(parents=True, exist_ok=True)
-        self._results['subjects_dir'] = str(subjects_dir)
+        self._results["subjects_dir"] = str(subjects_dir)
 
-        orig_subjects_dir = Path(self.inputs.freesurfer_home) / 'subjects'
+        orig_subjects_dir = Path(self.inputs.freesurfer_home) / "subjects"
 
         # Source is target, so just quit
         if subjects_dir == orig_subjects_dir:
@@ -726,12 +785,12 @@ class BIDSFreeSurferDir(SimpleInterface):
 
         spaces = list(self.inputs.spaces)
         # Always copy fsaverage, for proper recon-all functionality
-        if 'fsaverage' not in spaces:
-            spaces.append('fsaverage')
+        if "fsaverage" not in spaces:
+            spaces.append("fsaverage")
 
         for space in spaces:
             # Skip non-freesurfer spaces and fsnative
-            if not space.startswith('fsaverage'):
+            if not space.startswith("fsaverage"):
                 continue
             source = orig_subjects_dir / space
             dest = subjects_dir / space
@@ -750,7 +809,10 @@ class BIDSFreeSurferDir(SimpleInterface):
                 try:
                     copytree(source, dest)
                 except FileExistsError:
-                    LOGGER.warning("%s exists; if multiple jobs are running in parallel"
-                                   ", this can be safely ignored", dest)
+                    LOGGER.warning(
+                        "%s exists; if multiple jobs are running in parallel"
+                        ", this can be safely ignored",
+                        dest,
+                    )
 
         return runtime
